#include "ui_flash.h"
#include "app_util.h"
#include "mem_manager.h"
#include "nrf_assert.h"
#include "nrf_delay.h"
#include "nrf_soc.h"
#include "softdevice_handler.h"

flash_param uiFlashParametrs;

static void format_last_page(void);
 
/** @brief Function init flash table and reserved address.
 *
 * 
 * 
 */
uint32_t ui_flash_table_init(void)
{
		uint32_t err_code = NRF_SUCCESS;
	//uint8_t data[4]={0x9E, 0xCA, 0xDC, 0x24};
		uiFlashParametrs.pg_size = NRF_FICR->CODEPAGESIZE;
    uiFlashParametrs.pg_num  = NRF_FICR->CODESIZE - COUNT_PAGE;  // Use  page in flash
	
		 //uint32_t * table_addr[MAX_TABLE_ADDR];
		// Start address:
		uiFlashParametrs.start_addr = (uint32_t *)(uiFlashParametrs.pg_size * uiFlashParametrs.pg_num);		
		uiFlashParametrs.end_addr = (uint32_t *)(uiFlashParametrs.pg_size * (uiFlashParametrs.pg_num+ COUNT_PAGE - 17)-1);
		uiFlashParametrs.table_addr = uiFlashParametrs.start_addr;
		uiFlashParametrs.num_page = (((uint32_t )uiFlashParametrs.end_addr + 1) - (uint32_t )uiFlashParametrs.start_addr)/1024;
	
	
		/*sd_flash_page_erase( uiFlashParametrs.pg_num);

    

    nrf_delay_ms(20);
		ui_flash_save_data(0,data,sizeof(data));*/

    //sd_flash_write(uiFlashParametrs.start_addr, data, sizeof(data));
	//	sd_flash_page_erase( uiFlashParametrs.pg_num);
		
		return err_code;
}

uint32_t	ui_flash_save(uint32_t num_block, uint8_t * p_in_array, uint16_t size)//    
{
	uint32_t err_code = NRF_SUCCESS;
	
	//err_code=sd_softdevice_disable();
   //APP_ERROR_CHECK(err_code);
	
		if(num_block > MAX_TABLE_ADDR)
			return err_code = NRF_ERROR_INVALID_ADDR;
		
		uint32_t  *p_address = uiFlashParametrs.table_addr + num_block*(MAX_SIZE_BLOCK/4) ; 
		uint32_t temp_value = 0;
		temp_value = (uint32_t)*(p_address);
		
		ui_flash_clear_block( num_block);
		ui_flash_save_data(num_block,p_in_array,size);
		
		if(temp_value == FLASH_EMPTY_MASK)
			ui_flash_save_data(num_block,p_in_array,size);
		else
		{
			ui_flash_clear_block( num_block);			
			ui_flash_save_data(num_block,p_in_array,size);
		}
		//err_code=sd_softdevice_enable (NRF_CLOCK_LFCLKSRC_XTAL_20_PPM,NULL);//ble_evt_dispatch
   // APP_ERROR_CHECK(err_code);
	return err_code;
}

uint32_t ui_flash_verify(uint32_t num_block)//     
{
	uint32_t err_code = NRF_SUCCESS;
		if(num_block > MAX_TABLE_ADDR)
			return err_code = NRF_ERROR_INVALID_ADDR; //NRF_ERROR_INVALID_DATA
		
		uint32_t  *p_address = uiFlashParametrs.table_addr + num_block*(MAX_SIZE_BLOCK/4) ; 
		uint32_t temp_value = 0;
		temp_value = (uint32_t)*(p_address);
		
		if(temp_value == FLASH_EMPTY_MASK)
			err_code = NRF_ERROR_INVALID_DATA;
		
		return err_code;
}

/** @brief Function    flash memory.
 *
 * 
 * 
 */
uint32_t	ui_flash_clear_block(uint32_t num_block)
{
	uint32_t err_code = NRF_SUCCESS;
		if(num_block > MAX_TABLE_ADDR)
			return err_code = NRF_ERROR_INVALID_ADDR;
		uint32_t  *p_address = uiFlashParametrs.table_addr + num_block*(MAX_SIZE_BLOCK/4) ;
		uint32_t temp_value = 0;
		uint32_t temp_page = 0;
		uint32_t * fix_address =  p_address + (MAX_SIZE_BLOCK /4); 
		uint32_t temp_mas[256];
		
		temp_page = ui_get_page_for_Block(num_block); //     
		
		ui_read_all_data_from_page(temp_page,temp_mas); //     
		
	
		ui_flash_format_current_page(temp_page); // 
	
		uint32_t * addr = uiFlashParametrs.start_addr;	//        
		addr += (temp_page * 0x100);
		
		for(uint16_t i = 0; i< 256;i++) //        
		{
				
			if(addr != p_address )
			{				
				flash_word_write(addr, temp_mas[i]);				
				//sd_flash_write(p_address, &temp_mas[i],1);//don`t work
				
				addr++;
			}else{
			
				addr += (MAX_SIZE_BLOCK/4);
				i += (MAX_SIZE_BLOCK/4)-1;
			}
		}
		
	return err_code;
	
}

/** @brief Function update flash memory.
 *
 * 
 * 
 */
uint32_t	ui_flash_update(uint32_t num_block, uint8_t * p_in_array, uint16_t size)
{
	uint32_t err_code = NRF_SUCCESS;
		if(num_block > MAX_TABLE_ADDR)
			return err_code = NRF_ERROR_INVALID_ADDR;
		uint32_t  *p_address = uiFlashParametrs.table_addr + num_block*(MAX_SIZE_BLOCK/4) ;
		uint32_t temp_value = 0;
		uint32_t temp_page = 0;
		uint32_t * fix_address =  p_address + (MAX_SIZE_BLOCK /4); 
		uint32_t temp_mas[256];
		
		temp_page = ui_get_page_for_Block(num_block); //     
		
		ui_read_all_data_from_page(temp_page,temp_mas); //     
		
		ui_flash_format_current_page(temp_page); // 
		
		uint32_t * addr = uiFlashParametrs.start_addr;	//        
		addr += (temp_page * 0x100);
		
		for(uint16_t i = 0; i< 256;i++) //        
		{
				//temp_mas[i]=flash_read(*addr);
			if(addr != p_address )
			{
				flash_word_write(addr, temp_mas[i]);
				//sd_flash_write(p_address, &temp_mas[i],1);
				addr++;
			}else{
			
				addr += (MAX_SIZE_BLOCK/4);
				i += (MAX_SIZE_BLOCK/4)-1;
			}
		}
		
		ui_flash_save_data(num_block,p_in_array,size); //    
		
	return err_code;
	
}
/** @brief Function save data of block flash memory.
 *
 * @param[in] number of table block.
 * @param[in] pointer to array.
	* @param[in] size of array  .
 */
uint32_t ui_flash_save_data(uint32_t num_block, uint8_t * p_in_array, uint16_t size)
{
	uint32_t err_code = NRF_SUCCESS;
		if(num_block > MAX_TABLE_ADDR)
			return err_code = NRF_ERROR_INVALID_ADDR;
		
	uint32_t  *p_address = uiFlashParametrs.table_addr + num_block*(MAX_SIZE_BLOCK/4) ;
	
	uint32_t temp_value = 0;
	uint32_t * fix_address =  p_address + (MAX_SIZE_BLOCK /4); 
		
	for (uint16_t i = 0; i < size; i+=4)
    {
				temp_value = ( (((uint32_t)((uint8_t *)p_in_array)[i]) << 0) | (((uint32_t)((uint8_t *)p_in_array)[i+1]) << 8) |
										 (((uint32_t)((uint8_t *)p_in_array)[i+2]) << 16)  | (((uint32_t)((uint8_t *)p_in_array)[i+3]) << 24 ));
        flash_word_write(p_address, temp_value);
				//	sd_flash_write(p_address, temp_mas ,sizeof(temp_mas));
        p_address++;
					if((p_address > uiFlashParametrs.end_addr) || (p_address >  fix_address))
						return err_code = NRF_ERROR_INVALID_ADDR;
    } 
	return err_code;
}
/** @brief Function read data of page flash memory to mas from fixed number page.
 *
 * @param[in] number of flash page.
 * @param[out] set data to pointer array.
 */
uint32_t ui_read_all_data_from_page(uint32_t num_page, uint32_t * p_out_array)
{
	uint32_t err_code = NRF_SUCCESS;
	//uint32_t temp_mas[256];
	uint32_t * addr = uiFlashParametrs.start_addr;	//        
	addr += (num_page * 0x100);
	for(uint16_t i = 0; i< 256;i++)
	{
			//temp_mas[i]=flash_read(*addr);
		p_out_array[i]= (uint32_t)*(addr);//flash_read(addr);
		addr++;
	}
	return err_code;
}

/** @brief Function read data of page flash memory to mas from fixed number page.
 *
 * @param[in] number of flash page.
 * @param[out] set data to pointer array.
 */
uint32_t ui_falsh_read_data_from_block(uint32_t num_block, uint8_t * p_out_array, uint16_t size)
{
	
	uint32_t err_code = NRF_SUCCESS;
		if(num_block > MAX_TABLE_ADDR)
			return err_code = NRF_ERROR_INVALID_ADDR;
	//uint32_t temp_mas[256];
	uint32_t * addr = uiFlashParametrs.table_addr + num_block*(MAX_SIZE_BLOCK/4) ;	//        
	
	uint32_t temp_value = 0;
	uint32_t * fix_address =  addr + (MAX_SIZE_BLOCK /4); 
	for (uint16_t i = 0; i < size; i+=4)
    {
				temp_value = (uint32_t)*(addr);
				uint32_encode(temp_value,&p_out_array[i]);
        
       addr++;
					if((addr > uiFlashParametrs.end_addr) || (addr >  fix_address))
						return err_code = NRF_ERROR_INVALID_ADDR;
    }		
	return err_code;
}
/** @brief Function for get a page for create block.
 *
 * @param[in] number of table block.
 * @param[out] number of page to start adress.
 */
uint32_t ui_get_page_for_Block(uint32_t  num_block)
{
	uint32_t  *table_addr = uiFlashParametrs.table_addr + num_block*(MAX_SIZE_BLOCK/4) ;	
	uint32_t page = 0;
	//uint32_t page1 = 0;
	//page1 = 0x400;
	uint32_t  temp_addr =  (uint32_t)table_addr - (uint32_t)uiFlashParametrs.start_addr;
	page = temp_addr / 0x400;
	//page +=1;
	return page;
}
void ui_format_all_flash(void)
{
		
  uint32_t * addr = uiFlashParametrs.start_addr;	//     
 
  while (addr< uiFlashParametrs.end_addr) //    0400(1024 ) 
  {
		flash_page_erase(addr);
		
		addr += 0x100;
  }  

}

/** @brief Function for format current page.
 *
 * @param[in] number of page.
 * 
 */
void ui_flash_format_current_page(uint32_t num_page)
{	
  uint32_t * addr = uiFlashParametrs.start_addr;	//        
	addr += (num_page * 0x100);
	flash_page_erase(addr);		
	
}

static void format_last_page(void)
{	
  uint32_t * addr = (uint32_t *)0x0003BC00;	//        
	flash_page_erase(addr);		
}
/** @brief Function for erasing a page in flash.
 *
 * @param page_address Address of the first word in the page to be erased.
 */
 void flash_page_erase(uint32_t * page_address)
{
    // Turn on flash erase enable and wait until the NVMC is ready:
    NRF_NVMC->CONFIG = (NVMC_CONFIG_WEN_Een << NVMC_CONFIG_WEN_Pos);

    while (NRF_NVMC->READY == NVMC_READY_READY_Busy)
    {
        // Do nothing.
    }

    // Erase page:
    NRF_NVMC->ERASEPAGE = (uint32_t)page_address;

    while (NRF_NVMC->READY == NVMC_READY_READY_Busy)
    {
        // Do nothing.
    }

    // Turn off flash erase enable and wait until the NVMC is ready:
    NRF_NVMC->CONFIG = (NVMC_CONFIG_WEN_Ren << NVMC_CONFIG_WEN_Pos);

    while (NRF_NVMC->READY == NVMC_READY_READY_Busy)
    {
        // Do nothing.
    }
}

uint32_t flash_read(uint32_t * address)
{
        return ((__IO uint32_t) address);
}

/** @brief Function for filling a page in flash with a value.
 *
 * @param[in] address Address of the first word in the page to be filled.
 * @param[in] value Value to be written to flash.
 */
 void flash_word_write(uint32_t * address, uint32_t value)
{
    // Turn on flash write enable and wait until the NVMC is ready:
    NRF_NVMC->CONFIG = (NVMC_CONFIG_WEN_Wen << NVMC_CONFIG_WEN_Pos);

    while (NRF_NVMC->READY == NVMC_READY_READY_Busy)
    {
        // Do nothing.
    }

    *address = value;

    while (NRF_NVMC->READY == NVMC_READY_READY_Busy)
    {
        // Do nothing.
    }

    // Turn off flash write enable and wait until the NVMC is ready:
    NRF_NVMC->CONFIG = (NVMC_CONFIG_WEN_Ren << NVMC_CONFIG_WEN_Pos);

    while (NRF_NVMC->READY == NVMC_READY_READY_Busy)
    {
        // Do nothing.
    }
}
